const { Select, prompt } = require('enquirer');
const data = require('./../utils/data');
const Helper = require('../utils/helpers');
const { dim } = require('chalk');
const event = require('../utils/event');
let Table = require('cli-table');
const { selectProjectForm, confirmToggle } = require('./common');
const ElmBaseClass = require('./BaseClass');

class Team extends ElmBaseClass {
	constructor() {
		super();
		this.setServiceTitle('projectTeamMembers', '👥 Project Team Members');
	}

	setServiceTitle(key, title) {
		super.setServiceTitle(key, title);
	}

	async initialize() {
		return await this.teamMainMenu();
	}

	async teamMainMenu() {
		return await new Promise(async (resolve, reject) => {
			const actionOptions = {
				addNewMember: '📦 Add New Member',
				// listMembers: '📄 Project Members List ',
				deleteMember: '❗️ Delete Member',
				back: 'Back'
			};
			const prompt = new Select({
				name: 'action',
				message: 'What you want to do? - Team Members',
				choices: Object.values(actionOptions)
			});
			await prompt.run().then(async answer => {
				switch (answer) {
					case actionOptions.addNewMember:
						resolve(await this.addNewMember());
						break;
					case actionOptions.deleteMember:
						resolve(await this.deleteMember());
						break;
					case actionOptions.listMembers:
						resolve(await this.listMembers());
						break;
					case actionOptions.back:
						resolve({ success: false, action: event.back });
						break;
				}
			});
		});
	}

	async addNewMember() {
		return await new Promise(async resolve => {
			let projects = await data.getProjects();
			const selectProject = await selectProjectForm(projects);

			let responses = await prompt([
				{
					type: 'input',
					name: 'memberName',
					message: 'Please enter the member name',
					validate: Helper.getValidate()
				},
				{
					type: 'input',
					name: 'memberTeamEmail',
					message: 'Please enter the member `Teams` email',
					validate: Helper.getValidate()
				},
				{
					type: 'select',
					multiple: true,
					name: 'role',
					message: `Member role ${dim(
						'(you can select multiple by select then pressing space-bar)'
					)}`,
					choices: [
						{ name: 'Developer', value: 'developer' },
						{ name: 'Designer', value: 'designer' },
						{ name: 'Tester', value: 'tester' },
						{ name: 'Manager', value: 'manager' }
					],
					validate: Helper.getValidate()
				}
			]);

			let selectedProject = projects.filter(
				project => project.uuid === selectProject.project.uuid
			)[0];
			if (selectedProject.members === undefined) {
				selectedProject.members = [];
			}
			selectedProject.members?.push({
				name: responses.memberName,
				email: responses.memberTeamEmail,
				username: Helper.extractUsername(responses.memberTeamEmail),
				role: responses.role,
				uuid: Helper.uuidGenerator()
			});
			let projectsWithoutSelectedProject = projects.filter(
				project => project.uuid !== selectProject.project.uuid
			);
			projectsWithoutSelectedProject.push(selectedProject);
			await data.mainStore.set(
				'projects',
				projectsWithoutSelectedProject
			);

			resolve({
				success: true,
				message: `${responses.memberName} added successfully to project ${selectedProject.name}`,
				action: event.addNewMember
			});
		});
	}

	async listMembers() {
		return await new Promise(async resolve => {
			let projects = await data.getProjects();
			const selectProject = await selectProjectForm(projects);
			const table = new Table({
				head: ['Name', 'Username', 'email', 'Roles']
			});
			let selectedProject = projects.filter(
				project => project.uuid === selectProject.project.uuid
			)[0];
			if (selectedProject.members === undefined) {
				selectedProject.members = [];
			}
			selectedProject.members.forEach(member => {
				console.log();
				table.push([
					member.name,
					member.username,
					member.email,
					Helper.objToString(member.role)
				]);
			});

			console.log(table.toString());
			let answers = await prompt([
				{
					type: 'toggle',
					name: 'sure',
					message: 'Back to Main menu?',
					initial: true,
					enabled: 'Yes',
					disabled: 'No'
				}
			]);
			if (answers.sure) {
				resolve({ success: false, action: event.back });
			}
		});
	}

	async deleteMember() {
		return await new Promise(async resolve => {
			function resolveError(msg = '', action = event.deleteMember) {
				resolve({
					success: false,
					message: msg,
					action: action
				});
			}

			let projects = await data.getProjects();
			const selectProject = await selectProjectForm(projects);

			if (selectProject.project.uuid === 'cancel') {
				return resolveError('', event.back);
			}

			let selectedProject = projects.filter(
				project => project.uuid === selectProject.project.uuid
			)[0];
			let projectMembers = [];

			projectMembers = selectedProject?.members ?? [];
			if (projectMembers === undefined || projectMembers.length === 0) {
				return resolveError('No members in this project');
			}

			let cancelObject = {
				name: 'cancel',
				role: 'cancel',
				uuid: 'cancel'
			};
			projectMembers.filter(member => member.uuid === cancelObject.uuid)
				.length === 0 && projectMembers.push(cancelObject);

			const selectMember = await prompt({
				type: 'select',
				name: 'member',
				message: 'Please select the member',
				choices: projectMembers.map(member => {
					return {
						value: member.uuid,
						name: member.name,
						message: member.name
					};
				}),
				result: function () {
					return {
						name: this.focused.name,
						uuid: this.focused.value
					};
				}
			});
			if (selectMember.member.uuid === 'cancel') {
				return resolveError();
			}
			let selectedMemberRole = projectMembers.filter(
				member => member.uuid === selectMember.member.uuid
			)[0].role;
			if (selectedMemberRole.includes('owner')) {
				return resolveError('you can not remove the owner !');
			}

			let confirmAction = await confirmToggle();
			if (!confirmAction.sure) {
				return resolveError();
			}

			selectedProject.members = projectMembers.filter(
				member => member.uuid !== selectMember.member.uuid
			);

			let projectsWithoutSelectedProject = projects.filter(
				project => project.uuid !== selectProject.project.uuid
			);
			projectsWithoutSelectedProject.push(selectedProject);
			await data.mainStore.set(
				'projects',
				projectsWithoutSelectedProject
			);
			resolve({
				success: true,
				message: `${selectMember.member.name} deleted successfully from project ${selectedProject.name}`,
				action: event.deleteMember
			});
		});
	}
}

module.exports = Team;
