#!/usr/bin/env node

/**
 * elm-helper-cli
 *
 * @author Abdulrhman Alfulayt <https://twitter.com/Abdualrhman>
 */

const init = require('./utils/init');
const cli = require('./utils/cli');
const log = require('./utils/log');
const data = require('./utils/data');
const { Select } = require('enquirer');
const alert = require('cli-alerts');
const { addUserInfo } = require('./actions_operations/user_info');
const path = require('path');
const input = cli.input;
const flags = cli.flags;
const { clear, debug } = flags;
const QuestionsGenerator = require('./actions_operations/QuestionsGenerator');
const Project = require('./actions_operations/projects');
const events = require('./utils/event');

class ElmHelperCli {
	actionOptions = [];

	constructor() {
		this.appInit();
	}

	appInit() {
		init({ clear });
		input.includes(`help`) && cli.showHelp(0);
		debug && log(flags);
	}

	async initialize() {
		let mainUser = await data.getMainUser();
		if (mainUser == null) {
			let result = await addUserInfo();
			await this.startOver(result);
		}
		await this.mainMenu();
	}

	async startOver(result) {
		this.appInit();
		this.showMsg(result);
		await this.mainMenu();
	}

	setAction(action) {
		this.actionOptions.push(action);
	}

	setExitAction() {
		if (
			this.actionOptions.filter(
				action => action.service_title.key === 'exit'
			).length === 0
		) {
			this.actionOptions.push({
				service_title: {
					key: 'exit',
					value: '✖️ Exit'
				},
				initialize: function () {
					alert({
						type: 'info',
						msg: 'GoodBye :( '
					});
					process.exit(0);
				}
			});
		}
	}

	async mainMenu() {
		this.setExitAction();
		const prompt = new Select({
			name: 'action',
			message: 'What you want to do?',
			choices: this.actionOptions.map(action => {
				return {
					name: action.service_title.key,
					message: action.service_title.value
				};
			})
		});

		prompt
			.run()
			.then(async answer => {
				let action = this.actionOptions.filter(
					action => action.service_title.key === answer
				)[0];
				let result = await action.initialize();
				debug && console.log(result);
				return this.startOver(result);
			})
			.catch(console.error);
	}

	showMsg(result) {
		if (result.action === events.back) return;
		alert({
			type: result.success ? 'success' : 'error',
			msg: result.message ? result.message : 'something went wrong'
		});
	}
}

const setMeetingMomAction = () => {
	let meetingMom = new QuestionsGenerator(
		'Generate Meeting MOM',
		'meeting_mom'
	);
	const meetingMomPath = path.join(__dirname, './questions/meeting_mom.json');
	meetingMom.setPath(meetingMomPath);
	return meetingMom;
};
const setKpiAction = () => {
	let kpi = new QuestionsGenerator('Register KPI Evidence', 'KpiEvidence');
	const kpiPath = path.join(__dirname, './questions/Kpi.json');
	kpi.setPath(kpiPath);
	return kpi;
};

const setProjectAction = () => {
	return new Project();
};

(async () => {
	let elmHelper = new ElmHelperCli();
	elmHelper.setAction(setKpiAction());
	elmHelper.setAction(setMeetingMomAction());
	elmHelper.setAction(setProjectAction());
	await elmHelper.initialize();
})();
